#include <Keyboard.h>

const short BUTTON_RELEASE_STATE = HIGH;
const short BUTTON_PUSH_STATE = LOW;

const short INBOUND_PIN_COUNT = 4;

// Pin IDs
const short INBOUND_X = 5;
const short INBOUND_SPACE = 6;
const short INBOUND_ONE = 7;
const short INBOUND_CTRLZ = 8;
const short INBOUND_PINS[] = {INBOUND_X, INBOUND_SPACE, INBOUND_ONE, INBOUND_CTRLZ};

const short OUTBOUND = 10;

// Pin must maintain state for this many milliseconds.
const short DEBOUNCE_DURATION = 20;

// I'm treating these arrays as maps, since the keys to them are the integer
// pin ID. Arduinos have up to 14 pins so an array of size 14 should hold all
// of your INBOUND_* pins.
short last_state[14];
unsigned long debounce_millis[14];
int debounce_state[14];

void setup()
{
    Keyboard.begin();
    pinMode(OUTBOUND, OUTPUT);
    digitalWrite(OUTBOUND, BUTTON_PUSH_STATE);

    for (short index = 0; index < INBOUND_PIN_COUNT; index += 1)
    {
        short pin_id = INBOUND_PINS[index];
        // INPUT_PULLUP helped me fix unstable / noisy digital reads.
        pinMode(pin_id, INPUT_PULLUP);
        last_state[pin_id] = BUTTON_RELEASE_STATE;
        debounce_state[pin_id] = BUTTON_RELEASE_STATE;
        debounce_millis[pin_id] = millis();
    }
    delay(3000);
}

void manage_pin(short pin_id)
{
    unsigned long now = millis();
    short pin_state = digitalRead(pin_id);
    if (pin_state != debounce_state[pin_id])
    {
        debounce_millis[pin_id] = now;
        debounce_state[pin_id] = pin_state;
        return;
    }

    if (now - debounce_millis[pin_id] < DEBOUNCE_DURATION)
    {
        return;
    }

    // PIN IS DEBOUNCED BEYOND THIS POINT

    if (pin_state == last_state[pin_id])
    {
        return;
    }

    last_state[pin_id] = pin_state;

    if (pin_state != BUTTON_PUSH_STATE)
    {
        return;
    }

    switch(pin_id)
    {
        case INBOUND_X:
            // Serial.println("PRESSING X");
            Keyboard.write('X');
            break;
        case INBOUND_SPACE:
            // Serial.println("PRESSING SPACE");
            Keyboard.write(' ');
            break;
        case INBOUND_ONE:
            // Serial.println("PRESSING ONE");
            Keyboard.write('1');
            break;
        case INBOUND_CTRLZ:
            // Serial.println("PRESSING CTRLZ");
            Keyboard.press(KEY_LEFT_CTRL);
            Keyboard.press('z');
            delay(100);
            Keyboard.release(KEY_LEFT_CTRL);
            Keyboard.release('z');
            break;
    }
}

void loop()
{
    // digitalWrite(OUTBOUND, BUTTON_PUSH_STATE);
    for (int index = 0; index < INBOUND_PIN_COUNT; index += 1)
    {
        short pin_id = INBOUND_PINS[index];
        manage_pin(pin_id);
    }
}
